@@ -2,8 +2,8 @@
  * FILE:    memory.c
  * AUTHORS:  Isidor Kouvelas / Colin Perkins / Mark Handley / Orion Hodson
  *
- * $Revision: 1.19 $
- * $Date: 2001/04/01 22:18:46 $
+ * $Revision: 1.20 $
+ * $Date: 2002/04/01 20:19:01 $
  *
  * Copyright (c) 1995-2000 University College London
  * All rights reserved.
@@ -498,6 +498,9 @@ _xrealloc(void *p, unsigned size, const char *filen, int line)
                         debug_msg("realloc failed\n");
                         return NULL;
                 }
+		/* update ch/p */
+		ch = (chk_header*)m->addr;
+		p = (void*)(ch+1);
                 /* Update table */
                 free(m->filen);
                 m->filen  = (char *) strdup(filen);
