@@ -132,7 +132,7 @@ socket_error(const char *msg, ...)
 #define WSERR(x) {#x,x}
 	struct wse {
 		char  errname[20];
-		int errno;
+		int my_errno;
 	};
 	struct wse ws_errs[] = {
 		WSERR(WSANOTINITIALISED), WSERR(WSAENETDOWN),     WSERR(WSAEACCES),
@@ -147,7 +147,7 @@ socket_error(const char *msg, ...)
 	
 	int i, e = WSAGetLastError();
 	i = 0;
-	while(ws_errs[i].errno && ws_errs[i].errno != e) {
+	while(ws_errs[i].my_errno && ws_errs[i].my_errno != e) {
 		i++;
 	}
 	va_start(ap, msg);
@@ -176,6 +176,7 @@ socket_error(const char *msg, ...)
 
 /* winsock_versions_setsockopt tries 1 winsock version of option 
 * optname and then winsock 2 version if that failed.
+* note: setting the TTL never fails, so we have to try both.
 */
 
 static int
@@ -188,6 +189,7 @@ winsock_versions_setsockopt(SOCKET s, int level, int optname, const char FAR * o
 		break;
 	case IP_MULTICAST_TTL:
 		success = setsockopt(s, level, WS1_IP_MULTICAST_TTL, optval, optlen);
+		success = setsockopt(s, level, optname, optval, optlen);
 		break;
 	case IP_MULTICAST_LOOP:
 		success = setsockopt(s, level, WS1_IP_MULTICAST_LOOP, optval, optlen);
@@ -251,9 +253,12 @@ static int udp_addr_valid4(const char *dst)
 
 static socket_udp *udp_init4(const char *addr, const char *iface, uint16_t rx_port, uint16_t tx_port, int ttl)
 {
-	int                 	 reuse = 1;
+	int                 	 reuse = 1, udpbufsize=1048576;
 	struct sockaddr_in  	 s_in;
 	struct in_addr		 iface_addr;
+#ifdef WIN32
+      int recv_buf_size = 65536;
+#endif
 	socket_udp         	*s = (socket_udp *)malloc(sizeof(socket_udp));
 	s->mode    = IPv4;
 	s->addr    = NULL;
@@ -281,6 +286,14 @@ static socket_udp *udp_init4(const char *addr, const char *iface, uint16_t rx_po
 	s->fd = socket(AF_INET, SOCK_DGRAM, 0);
 	if (s->fd < 0) {
 		socket_error("socket");
+	if (SETSOCKOPT(s->fd, SOL_SOCKET, SO_SNDBUF, (char *) &udpbufsize, sizeof(udpbufsize)) != 0) {
+		socket_error("setsockopt SO_SNDBUF");
+		return NULL;
+	}
+	if (SETSOCKOPT(s->fd, SOL_SOCKET, SO_RCVBUF, (char *) &udpbufsize, sizeof(udpbufsize)) != 0) {
+		socket_error("setsockopt SO_RCVBUF");
+		return NULL;
+	}
 		return NULL;
 	}
 	if (SETSOCKOPT(s->fd, SOL_SOCKET, SO_REUSEADDR, (char *) &reuse, sizeof(reuse)) != 0) {
@@ -349,7 +362,8 @@ static void udp_exit4(socket_udp *s)
 	free(s);
 }
 
-static int udp_send4(socket_udp *s, char *buffer, int buflen)
+static inline int 
+udp_send4(socket_udp *s, char *buffer, int buflen)
 {
 	struct sockaddr_in	s_in;
 	
@@ -364,6 +378,31 @@ static int udp_send4(socket_udp *s, char *buffer, int buflen)
 	return sendto(s->fd, buffer, buflen, 0, (struct sockaddr *) &s_in, sizeof(s_in));
 }
 
+#ifndef WIN32
+static inline int 
+udp_sendv4(socket_udp *s, struct iovec *vector, int count)
+{
+	struct msghdr		msg;
+	struct sockaddr_in	s_in;
+	
+	assert(s != NULL);
+	assert(s->mode == IPv4);
+	
+	s_in.sin_family      = AF_INET;
+	s_in.sin_addr.s_addr = s->addr4.s_addr;
+	s_in.sin_port        = htons(s->tx_port);
+
+	msg.msg_name       = &s_in;
+	msg.msg_namelen    = sizeof(s_in);
+	msg.msg_iov        = vector;
+	msg.msg_iovlen     = count;
+	msg.msg_control    = 0;
+	msg.msg_controllen = 0;
+	msg.msg_flags      = 0;
+	return sendmsg(s->fd, &msg, 0);
+}
+#endif
+
 static const char *udp_host_addr4(void)
 {
 	static char    		 hname[MAXHOSTNAMELEN];
@@ -553,6 +592,41 @@ static int udp_send6(socket_udp *s, char *buffer, int buflen)
 #endif
 }
 
+#ifndef WIN32
+static int 
+udp_sendv6(socket_udp *s, struct iovec *vector, int count)
+{
+#ifdef HAVE_IPv6
+	struct msghdr		msg;
+	struct sockaddr_in6	s_in;
+	
+	assert(s != NULL);
+	assert(s->mode == IPv6);
+	
+	memset((char *)&s_in, 0, sizeof(s_in));
+	s_in.sin6_family = AF_INET6;
+	s_in.sin6_addr   = s->addr6;
+	s_in.sin6_port   = htons(s->tx_port);
+#ifdef HAVE_SIN6_LEN
+	s_in.sin6_len    = sizeof(s_in);
+#endif
+	msg.msg_name       = &s_in;
+	msg.msg_namelen    = sizeof(s_in);
+	msg.msg_iov        = vector;
+	msg.msg_iovlen     = count;
+	msg.msg_control    = 0;
+	msg.msg_controllen = 0;
+	msg.msg_flags      = 0;
+	return sendmsg(s->fd, &msg, 0);
+#else
+	UNUSED(s);
+	UNUSED(vector);
+	UNUSED(count);
+	return -1;
+#endif
+}
+#endif
+
 static const char *udp_host_addr6(socket_udp *s)
 {
 #ifdef HAVE_IPv6
@@ -715,6 +789,20 @@ int udp_send(socket_udp *s, char *buffer, int buflen)
 	return -1;
 }
 
+
+#ifndef WIN32
+int         
+udp_sendv(socket_udp *s, struct iovec *vector, int count)
+{
+	switch (s->mode) {
+	case IPv4 : return udp_sendv4(s, vector, count);
+	case IPv6 : return udp_sendv6(s, vector, count);
+	default   : abort(); /* Yuk! */
+	}
+	return -1;
+}
+#endif
+
 /**
  * udp_recv:
  * @s: UDP session.
@@ -836,3 +924,4 @@ int udp_fd(socket_udp *s)
 	return 0;
 }
 
+
